/*
 * Gui.cpp
 * Copyright (C) Tobias Groenhagen 2009
 *
 */

#include "Gui.hpp"
#include <QtGui>

//#####################################################################

Gui::Gui(std::string path, int argc, char *argv[])
{
  app=new QApplication(argc, argv);
  winf=new Info();
  win=new Window(path,winf);
}

Gui::~Gui()
{
  win->close();
  winf->close();
  delete(winf);
  delete(win);
  delete(app);
}

int Gui::show()
{
  win->show();
  return app->exec();
}

//#####################################################################

Window::Window(std::string path, Info *iwin, QWidget *parent)
: QDialog(parent), iwin(iwin)
{
  setupUi(this);
  proc = new ProcessList(path);
  listm = new ProcessListModel(proc);
  treem = new ProcessTreeModel(proc);
  viewList->setModel(listm);
  viewTree->setModel(treem);
  viewList->setHeaderHidden( false );
  viewTree->setHeaderHidden( false );

  connect(btExit,   SIGNAL(clicked()), this, SLOT(ende()));

  connect(viewList, SIGNAL(doubleClicked(const QModelIndex&)),
            this, SLOT(showListInfo(const QModelIndex&)));

  connect(viewTree, SIGNAL(doubleClicked(const QModelIndex&)),
            this, SLOT(showTreeInfo(const QModelIndex&)));

}

Window::~Window()
{
  delete(listm);
  delete(treem);
  delete(proc);
}

void Window::ende()
{
  iwin->close();
  close();
}

void Window::showListInfo(const QModelIndex& index)
{
  if (index.isValid())
  {
    QVariant qv = listm->data(index,Qt::UserRole);
    int pid = qv.toInt();
    iwin->setProcess(proc->getProcess(pid));
    iwin->show();
  }
}


void Window::showTreeInfo(const QModelIndex& index)
{
  if (index.isValid())
  {
    QVariant qv = treem->data(index,Qt::UserRole);
    int pid = qv.toInt();
    Process *p=proc->getProcess(pid);
    iwin->setProcess(p);
    iwin->show();
  }
}

//####################################################################

Info::Info(QWidget *parent)
: QDialog(parent)
{
  setupUi(this);
  connect(btEcit,   SIGNAL(clicked()), this, SLOT(clean()));
}

void Info::setProcess(const Process *p)
{
  QString name = p->getName().c_str();

  QString pid;
  pid.setNum(p->getPID());

  QString cmd = p->getCMD().c_str();

  QString utime;
  utime.setNum(p->getUTime());

  QString stime;
  stime.setNum(p->getSTime());

  QString sdate;
  sdate.setNum(p->getStartTime());

  lcmd->setText(cmd);
  lpid->setText(pid);
  lname->setText(name);
  lutime->setText(utime);
  lstime->setText(stime);
  lsdate->setText(sdate);

}

void Info::clean(void)
{
  lcmd->setText(QString());
  lpid->setText(QString());
  lname->setText(QString());
  lutime->setText(QString());
  lstime->setText(QString());
  lsdate->setText(QString());
  close();
}

//####################################################################
//Baum
ProcessTreeModel::ProcessTreeModel(ProcessList *proc, QObject *parent)
: QAbstractListModel(parent), proc(proc)
{}

QModelIndex ProcessTreeModel::ProcessToIndex(
                                              const Process *p,
                                              int column
                                            ) const
{
  QModelIndex qmi;

  if (p == 0 || p->isRoot()) qmi= QModelIndex();
  unsigned int row = proc->getChildPosByPID(p->getPPID(),p->getPID());
  qmi= createIndex(row, column, const_cast<Process*>(p));

  return qmi;
}

Process* ProcessTreeModel::IndexToProcess
                     (const QModelIndex &index) const
{
  Process *rp = proc->getProcess(-1);
  if (index.isValid())
    rp=static_cast<Process*>(index.internalPointer());

  return rp;
}

QModelIndex ProcessTreeModel::index(
                                     int pos,
                                     int column,
                                     const QModelIndex &parent
                                   ) const
{
  const Process *p = IndexToProcess(parent);
  QModelIndex qmi;

  if(pos>=0 && column>=0)
  {
    Process *cp=proc->getChildProcessByPos(p->getPID(), pos);
    qmi = ProcessToIndex( cp, column );
    parentHash[qmi] = parent;
  }

  return qmi;
}

QModelIndex ProcessTreeModel::parent(const QModelIndex &child) const
{
  const Process *p = IndexToProcess(child);

  QModelIndex qmi;
  if (!p->isRoot()) qmi=parentHash[child];

  return qmi;
}

bool ProcessTreeModel::hasChildren( const QModelIndex &parent) const
{
  bool has=true;

  if (parent.isValid())
  {
    Process *p = IndexToProcess(parent);
    if(proc->getChildProcessLength(p->getPID()) == 0)
      has=false;
  }

  return has;
}

int ProcessTreeModel::rowCount(const QModelIndex &parent) const
{
  const Process *p = IndexToProcess(parent);
  int cnt=proc->getChildProcessLength(p->getPID());
  return cnt;
}

int ProcessTreeModel::columnCount(const QModelIndex &parent) const
{
  //parent.column();
  return 4;
}

QVariant ProcessTreeModel::data(
                                 const QModelIndex &index,
                                 int role
                               ) const
{
  QVariant qv;

  if( index.isValid() )
  {
    const Process *p = IndexToProcess(index);
    if(role == Qt::DisplayRole)
    {
      if (index.row() >= 0 || index.column() >= 0)
      {
        QString ret;
        switch(index.column())
        {
          case 0: ret= p->getName().c_str(); break;
          case 1: ret.setNum(p->getPID()); break;
          case 2: ret= p->getState(); break;
          case 3: ret.setNum(p->getNice()); break;
          default: ret=p->getLine().c_str();
        }
        qv = ret;
      }
    }
    else if(role == Qt::UserRole)
    {
      QString ret;
      ret.setNum(p->getPID());
      qv = ret;
    }
  }
  return qv;
}

QVariant ProcessTreeModel::headerData(
                                       int section,
                                       Qt::Orientation orientation,
                                       int role
                                     ) const
{
  QVariant qv;
  if(role== Qt::DisplayRole && orientation == Qt::Horizontal)
  {
    QString ret;
    switch(section)
    {
      case 0: ret="NAME"; break;
      case 1: ret="PID"; break;
      case 2: ret="STATUS"; break;
      case 3: ret="NICE"; break;
      default: ret="???";
    }
    qv=ret;
  }

  return qv;
}

//####################################################################
// Liste
ProcessListModel::ProcessListModel(ProcessList *proc, QObject *parent)
: QAbstractListModel(parent), proc(proc)
{}

int ProcessListModel::rowCount(const QModelIndex &parent) const
{
  parent.column();
  return proc->getPIDs().size();
}

int ProcessListModel::columnCount(const QModelIndex &parent) const
{
  if(parent.column()>0) return 0;
  return 5;
}

QVariant ProcessListModel::data(
                                 const QModelIndex &index,
                                 int role
                               ) const
{
  if(!index.isValid())
    return QVariant();

  if(index.row() < 0 || index.row() > rowCount())
    return QVariant();

  std::vector<Process> *list=proc->getProcesses();
  int pos=index.row();
  if(role == Qt::DisplayRole)
  {
    QString ret;
    switch(index.column())
    {
      case 0: ret.setNum(list->at(pos).getPID()); break;
      case 1: ret=list->at(pos).getName().c_str(); break;
      case 2: ret=list->at(pos).getState(); break;
      case 3: ret.setNum(list->at(pos).getNice()); break;
      case 4: ret.setNum(list->at(pos).getPPID()); break;
      default: ret=list->at(pos).getLine().c_str();
    }
    return ret;
  } else if(role == Qt::UserRole)
    {
      QString ret;
      ret.setNum(list->at(pos).getPID());
      return ret;
    }


  return QVariant();
}

QVariant ProcessListModel::headerData(
                                       int section,
                                       Qt::Orientation orientation,
                                       int role
                                     ) const
{
  QVariant ret;
  if(role== Qt::DisplayRole && orientation == Qt::Horizontal)
  {
    QString qs;
    switch(section)
    {
      case 0: qs="PID"; break;
      case 1: qs="NAME"; break;
      case 2: qs="STATUS"; break;
      case 3: qs="NICE"; break;
      case 4: qs="PPID"; break;
      default: qs="???";
    }
    ret=qs;
  }
  return ret;
}
